/*!
 * fine-mq v1.0.3
 * (c) 2018-present Honoré Nintunze <nintu.lab@gmail.com>
 * Released under the MIT License.
 */
'use strict';

function _classCallCheck(instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
}

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  return Constructor;
}

function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest();
}

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArrayLimit(arr, i) {
  var _arr = [];
  var _n = true;
  var _d = false;
  var _e = undefined;

  try {
    for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance");
}

var Mq =
/*#__PURE__*/
function () {
  function Mq() {
    var aliases = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    _classCallCheck(this, Mq);

    this.aliases = aliases;
    this.queries = {};
  }

  _createClass(Mq, [{
    key: "alias",
    value: function alias(_alias, query) {
      if (query) {
        this.aliases[_alias] = query;
        return this;
      }

      this.aliases = Object.assign({}, this.aliases, _alias);
      return this;
    }
  }, {
    key: "unalias",
    value: function unalias(alias) {
      var queryObject = this.queries[this.aliases[alias]];
      if (queryObject) this.off(alias);
      delete this.aliases[alias];
      return this;
    }
  }, {
    key: "on",
    value: function on(query, callback, context) {
      context = context || this;
      var res = this.queries[this.aliases[query] || query];
      var queryObject = res ? res : {
        handlers: [],
        mqMatcher: window.matchMedia(this.aliases[query] || query)
      };

      queryObject.listener = function () {
        queryObject.handlers.forEach(function (handler) {
          return handler.callback.call(handler.context, {
            matcher: queryObject.mqMatcher,
            alias: query
          });
        });
      };

      queryObject.mqMatcher.addListener(queryObject.listener);
      var handler = {
        callback: callback,
        context: context
      };
      queryObject.handlers.push(handler);
      this.queries[this.aliases[query] || query] = queryObject;
      callback.call(context, {
        matcher: queryObject.mqMatcher,
        alias: query
      }); // initial trigger

      return this;
    }
  }, {
    key: "off",
    value: function off(query, callback, context) {
      var _this = this;

      if (!arguments.length) {
        // remove all listeners
        Object.entries(this.queries).forEach(function (_ref) {
          var _ref2 = _slicedToArray(_ref, 2),
              key = _ref2[0],
              queryObject = _ref2[1];

          _this._removeQueryObject(queryObject, key);
        });
        return this;
      }

      if (!callback) {
        // remove all listeners for a query
        this._removeQueryObject(query);

        return this;
      } // remove a particular listener for a query


      var queryObject = this.queries[this.aliases[query] || query];

      if (queryObject) {
        var handlers = queryObject.handlers;
        handlers.forEach(function (handler, i) {
          var match = handler.callback === callback;
          if (match && context) match = handler.context === context;
          if (match) handlers.splice(i, 1);
        });
        if (!handlers.length) this._removeQueryObject(query);
      }

      return this;
    }
  }, {
    key: "_removeQueryObject",
    value: function _removeQueryObject(qObject, query) {
      if (typeof qObject === 'string') {
        query = qObject;
        qObject = undefined;
      }

      query = this.aliases[query] || query;
      var queryObject = qObject || this.queries[query];

      if (queryObject) {
        queryObject.mqMatcher.removeListener(queryObject.listener);
        delete this.queries[query];
      }
    }
  }]);

  return Mq;
}();

module.exports = Mq;
